package cz.jzitnik;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.SpriteLoader;
import cz.jzitnik.tui.ScreenRenderer;
import org.jline.terminal.Terminal;
import org.jline.terminal.TerminalBuilder;

import java.io.IOException;

public class Main {
    public static void main(String[] args) {
        try {
            // Set up terminal with JLine
            Terminal terminal = TerminalBuilder.terminal();
            terminal.enterRawMode();
            var spriteList = SpriteLoader.load();
            var screenRenderer = new ScreenRenderer(spriteList);
            var game = new Game();

            final boolean[] isRunning = {true};

            // Enable mouse tracking
            terminal.writer().print("\033[?1003h\n"); // Enable mouse move events
            terminal.writer().flush();

            Thread inputThread = new Thread(() -> {
                try {
                    while (isRunning[0]) {
                        int key = terminal.reader().read();

                        // Check for mouse event sequence
                        if (key == 27) { // ESC
                            key = terminal.reader().read();
                            if (key == '[') {
                                key = terminal.reader().read();
                                if (key == 'M') {
                                    int buttonCode = terminal.reader().read() - 32;
                                    int x = terminal.reader().read() - 32;
                                    int y = terminal.reader().read() - 32;

                                    if (buttonCode == 0) {
                                        game.handleMouseClick(x, y);
                                        throw new RuntimeException("sad");
                                        //screenRenderer.render(game.getWorld());
                                    }
                                }
                            }
                        }

                        // Handle keyboard input as before
                        switch (key) {
                            case 'a':
                                game.movePlayerLeft(screenRenderer);
                                screenRenderer.render(game.getWorld());
                                break;
                            case 'd':
                                game.movePlayerRight(screenRenderer);
                                screenRenderer.render(game.getWorld());
                                break;
                            case ' ':
                                game.movePlayerUp(screenRenderer);
                                screenRenderer.render(game.getWorld());
                                break;
                            case 'm':
                                game.mine(screenRenderer);
                                break;
                            case 'q':
                                System.out.println("Exiting game...");
                                isRunning[0] = false;
                                break;
                            default:
                                break;
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            });

            inputThread.start();

            // Game loop (rendering the game)
            while (isRunning[0]) {
                screenRenderer.render(game.getWorld());
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            // Disable mouse tracking on exit
            terminal.writer().print("\033[?1003l\n");
            terminal.writer().flush();

            terminal.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
