package cz.jzitnik.game;

import cz.jzitnik.game.sprites.Breaking;
import cz.jzitnik.tui.ScreenRenderer;
import lombok.Getter;
import java.util.ArrayList;
import java.util.List;

@Getter
public class Game {
    private List<Block>[][] world = new ArrayList[50][50];
    private Block player;
    private boolean mining;

    public Game() {
        for (int i = 0; i < 50; i++) {
            for (int j = 0; j < 50; j++) {
                world[i][j] = new ArrayList<>();
            }
        }

        Block steveBlock = new Block("steve", SpriteLoader.SPRITES.STEVE);
        world[10][2].add(new Block("grass", SpriteLoader.SPRITES.GRASS));
        player = steveBlock;

        for (int i = 0; i < 50; i++) {
            world[11][i].add(new Block("grass", SpriteLoader.SPRITES.GRASS));
            world[12][i].add(new Block("dirt", SpriteLoader.SPRITES.DIRT));
            world[13][i].add(new Block("dirt", SpriteLoader.SPRITES.DIRT));
            world[14][i].add(new Block("dirt", SpriteLoader.SPRITES.DIRT));

            for (int j = 15; j < 49; j++) {
                world[j][i].add(new Block("stone", SpriteLoader.SPRITES.STONE));
            }

            world[49][i].add(new Block("bedrock", SpriteLoader.SPRITES.BEDROCK));
        }

        for (int i = 0; i < world.length; i++) {
            for (int j = 0; j < world[i].length; j++) {
                if (world[i][j].isEmpty()) {
                    world[i][j].add(new Block("air", SpriteLoader.SPRITES.AIR, true)); // Fill with air
                }
            }
        }

        world[10][0].add(steveBlock);
    }

    private int[] getPlayerCords() {
        for (int i = 0; i < world.length; i++) {
            for (int j = 0; j < world[i].length; j++) {
                for (Block block : world[i][j]) {
                    if (block.getBlockId().equals("steve")) {
                        return new int[]{j, i};
                    }
                }
            }
        }
        return null;
    }

    public void movePlayerRight(ScreenRenderer screenRenderer) {
        int[] cords = getPlayerCords();

        if (world[cords[1]][cords[0] + 1].stream().anyMatch(block -> !block.isGhost())) {
            return;
        }

        world[cords[1]][cords[0] + 1].add(player);
        world[cords[1]][cords[0]].remove(player);


        new Thread(() -> {
            while (true) {
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                int[] cords2 = getPlayerCords();

                if (world[cords2[1] + 1][cords2[0]].stream().anyMatch(Block::isGhost)) {
                    world[cords2[1] + 1][cords2[0]].add(player);
                    world[cords2[1]][cords2[0]].remove(player);

                    screenRenderer.render(world);
                } else {
                    break;
                }
            }
        }).start();
    }

    public void movePlayerLeft(ScreenRenderer screenRenderer) {
        int[] cords = getPlayerCords();

        if (world[cords[1]][cords[0] - 1].stream().anyMatch(block -> !block.isGhost())) {
            return;
        }

        world[cords[1]][cords[0] - 1].add(player);
        world[cords[1]][cords[0]].remove(player);

        new Thread(() -> {
            while (true) {
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                int[] cords2 = getPlayerCords();

                if (world[cords2[1] + 1][cords2[0]].stream().anyMatch(Block::isGhost)) {
                    world[cords2[1] + 1][cords2[0]].add(player);
                    world[cords2[1]][cords2[0]].remove(player);

                    screenRenderer.render(world);
                } else {
                    break;
                }
            }
        }).start();
    }

    public void movePlayerUp(ScreenRenderer screenRenderer) {
        int[] cords = getPlayerCords();

        if (world[cords[1] - 1][cords[0]].stream().anyMatch(block -> !block.isGhost()) || world[cords[1] + 1][cords[0]].stream().anyMatch(Block::isGhost)) {
            return;
        }

        world[cords[1] - 1][cords[0]].add(player);
        world[cords[1]][cords[0]].remove(player);

        new Thread(() -> {
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            int[] cords2 = getPlayerCords();

            if (world[cords2[1] + 1][cords2[0]].stream().anyMatch(Block::isGhost)) {
                world[cords2[1] + 1][cords2[0]].add(player);
                world[cords2[1]][cords2[0]].remove(player);

                screenRenderer.render(world);
            }
        }).start();
    }

    public void mine(ScreenRenderer screenRenderer) {
        int[] cords = getPlayerCords();

        if (world[cords[1] + 1][cords[0]].stream().allMatch(Block::isGhost)) {
            return;
        }

        world[cords[1] + 1][cords[0]].clear();
        world[cords[1] + 1][cords[0]].add(new Block("air", SpriteLoader.SPRITES.AIR, true));
        screenRenderer.render(world);

        new Thread(() -> {
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            int[] cords2 = getPlayerCords();

            if (world[cords2[1] + 1][cords2[0]].stream().anyMatch(Block::isGhost)) {
                world[cords2[1] + 1][cords2[0]].add(player);
                world[cords2[1]][cords2[0]].remove(player);

                screenRenderer.render(world);
            }
        }).start();
    }

    public void handleMouseClick(int mouseX, int mouseY) {
        int[] cords = getPlayerCords();
        if (cords == null) return;

        int playerX = cords[0];
        int playerY = cords[1];
        int viewXRadius = 5;
        int viewUpRadius = 4;
        int viewDownRadius = 3;

        // Calculate visible area boundaries
        int startX = Math.max(0, playerX - viewXRadius);
        int endX = Math.min(world[0].length, playerX + viewXRadius + 1);
        int startY = Math.max(0, playerY - viewUpRadius);
        int endY = Math.min(world.length, playerY + viewDownRadius + 1);

        // Convert mouse coordinates to world coordinates
        int blockX = startX + (mouseX / 50);  // 50 chars wide per sprite
        int blockY = startY + (mouseY / 25);  // 25 lines high per sprite

        // Check if within bounds
        if (blockY >= startY && blockY < endY && blockX >= startX && blockX < endX) {
            List<Block> blocks = world[blockY][blockX];
            if (!blocks.isEmpty()) {
                Block topBlock = blocks.get(blocks.size() - 1);  // Get topmost block
                System.out.println("Mouse clicked on block: " + topBlock.getBlockId() +
                        " at World Coordinates: [" + blockX + ", " + blockY + "]");
            } else {
                System.out.println("No block at the clicked position.");
            }
        } else {
            System.out.println("Clicked outside the visible area.");
        }
    }
}